import { ConditionExpression } from './conditions';
import { DynamoMarshallerFor } from './marshalling';
import { Queryable } from './queryable';
import { DynamodbTableConfig, IndexDefinition, IndexDefinitions, TableDefinition } from './table-builder';
import { DynamoKeyTypes, DynamoObject, DynamoPrimitive, DynamoValueKeys, NestedKeyOf, NestedPick, Prettify, RecursivePartial, RequireAtLeastOne, UnionToIntersection } from './types';
type UpdateReturnValue = 'NONE' | 'ALL_OLD' | 'UPDATED_OLD' | 'ALL_NEW' | 'UPDATED_NEW';
declare const IfNotExistsKey: unique symbol;
type IfNotExists<O> = {
    readonly ifNotExists: O;
    readonly [IfNotExistsKey]: true;
};
export declare const ifNotExists: <O>(o: O) => IfNotExists<O>;
type SetOpts<A extends DynamoObject, RV extends UpdateReturnValue> = {
    readonly returnValue?: RV;
    readonly conditionExpression?: ConditionExpression<A>;
};
type PutOpts<A extends DynamoObject> = {
    readonly conditionExpression?: ConditionExpression<A>;
};
type PartSetResponse<A extends DynamoObject, RV extends UpdateReturnValue> = RV extends 'ALL_NEW' ? A : RV extends 'ALL_OLD' ? A : RV extends 'NONE' ? void : Partial<A>;
type SetResponse<A extends DynamoObject, RV extends UpdateReturnValue> = PartSetResponse<A, RV>;
type TransactWriteItem<I, CE> = {
    readonly item: I;
    readonly conditionExpression?: ConditionExpression<CE>;
};
type SetTarget<T, N extends boolean = false> = T | (N extends false ? never : IfNotExists<T>) | {
    readonly [K in keyof T]: SetTarget<T[K], true>;
};
type AttributeDynamoTypeMap<S extends DynamoKeyTypes> = S extends string ? 'S' : S extends number ? 'N' : S extends ArrayBuffer | ArrayBufferView ? 'B' : never;
type AttributeDefinitions<T extends DynamoObject, HK extends string, RK extends string | never> = (NonNullable<T[HK]> extends DynamoKeyTypes ? {
    readonly [k in HK]-?: AttributeDynamoTypeMap<NonNullable<T[HK]>>;
} : {}) & (NonNullable<T[RK]> extends DynamoKeyTypes ? {
    readonly [k in RK]-?: AttributeDynamoTypeMap<NonNullable<T[RK]>>;
} : {});
type IndexAttributes<T extends IndexDefinitions> = {
    readonly [k in keyof T]: T[k] extends IndexDefinition<infer A, infer HK, infer RK> ? AttributeDefinitions<A, HK, RK> : undefined;
}[keyof T];
type CreateTableAttributeDefinitions<T extends DynamoObject, HK extends string, RK extends string | never, Ixs extends IndexDefinitions> = Prettify<UnionToIntersection<AttributeDefinitions<T, HK, RK> & (IndexAttributes<Ixs> extends never ? {} : IndexAttributes<Ixs>)>>;
type CreateTableIndexDefinition = {
    readonly provisionedThroughput: ProvisionedThroughput;
};
type ProvisionedThroughput = {
    readonly read: number;
    readonly write: number;
};
type BillingMode = 'PROVISIONED' | 'PAY_PER_REQUEST';
type CreateProps<A extends DynamoObject, HK extends string, RK extends string | never, Ixs extends IndexDefinitions, BM extends BillingMode> = {
    readonly billingMode: BM;
    readonly keyDefinitions: CreateTableAttributeDefinitions<A, HK, RK, Ixs>;
} & (BM extends 'PROVISIONED' ? {
    readonly provisionedThroughput: ProvisionedThroughput;
    readonly indexDefinitions: {
        readonly [k in keyof Ixs]?: CreateTableIndexDefinition;
    };
} : {});
export type Table<A extends DynamoObject, HK extends string, RK extends string | never, Ixs extends IndexDefinitions> = {
    readonly createTable: <BM extends BillingMode>(props: CreateProps<A, HK, RK, Ixs, BM>) => Promise<void>;
    readonly deleteTable: () => Promise<void>;
    readonly get: <AA extends A, Keys extends NestedKeyOf<AA> = NestedKeyOf<AA>>(hk: Prettify<Pick<A, HK | RK>>, opts?: {
        readonly consistentRead?: boolean;
        readonly marshaller?: DynamoMarshallerFor<A>;
        readonly keys?: readonly Keys[];
    }) => Promise<NestedPick<AA, Keys> | null>;
    readonly batchGet: <Keys extends NestedKeyOf<A> = NestedKeyOf<A>>(hks: readonly Prettify<Pick<A, HK | RK>>[], opts?: {
        readonly consistentRead?: boolean;
        readonly keys?: readonly Keys[];
    }) => Promise<readonly NestedPick<A, Keys>[]>;
    readonly transactGet: (hks: readonly Prettify<Pick<A, HK | RK>>[]) => Promise<readonly A[]>;
    readonly put: <AA extends A>(a: AA, opts?: PutOpts<A>) => Promise<void>;
    readonly set: <RV extends UpdateReturnValue = 'NONE'>(key: Prettify<Pick<A, HK | RK>>, updates: RecursivePartial<SetTarget<Omit<A, HK | RK>>>, opts?: SetOpts<Omit<A, HK | RK>, RV>) => Promise<SetResponse<A, RV>>;
    readonly batchPut: (a: ReadonlyArray<A>) => Promise<void>;
    readonly batchDelete: (keys: ReadonlyArray<Pick<A, HK | RK>>) => Promise<void>;
    readonly transactPut: (a: ReadonlyArray<TransactWriteItem<A, A>>) => Promise<void>;
    readonly transactDelete: (keys: ReadonlyArray<TransactWriteItem<Pick<A, HK | RK>, A>>) => Promise<void>;
    readonly transactWrite: (args: RequireAtLeastOne<{
        readonly puts: ReadonlyArray<TransactWriteItem<A, A>>;
        readonly deletes: ReadonlyArray<TransactWriteItem<Pick<A, HK | RK>, A>>;
        readonly updates: ReadonlyArray<{
            readonly key: Pick<A, HK | RK>;
            readonly updates: RecursivePartial<SetTarget<Omit<A, HK | RK>>>;
            readonly conditionExpression?: ConditionExpression<A>;
        }>;
    }>) => Promise<void>;
    readonly delete: (hk: Pick<A, HK | RK>, opts?: {
        readonly conditionExpression?: ConditionExpression<A>;
    }) => Promise<void>;
    readonly indexes: Indexes<Ixs>;
};
export type QueryableTable<A extends DynamoObject, HK extends string, RK extends string, Ixs extends IndexDefinitions> = Table<A, HK, RK, Ixs> & Queryable<A, HK, RK>;
export type TableFactoryResult<A extends DynamoObject, HK extends string, RK extends string | undefined, Ixs extends IndexDefinitions> = RK extends undefined ? Table<A, HK, never, Ixs> : QueryableTable<A, HK, Exclude<RK, undefined>, Ixs>;
export type TableConfig<A extends Record<string, DynamoPrimitive>, HK extends DynamoValueKeys<A> & string, RK extends DynamoValueKeys<A> & string = never> = {
    readonly hashKey: HK;
    readonly sortKey?: RK;
};
type Index<ID> = ID extends IndexDefinition<infer T, infer PK, infer SK> ? Queryable<T, PK, SK> : never;
type Indexes<ID> = ID extends IndexDefinitions ? {
    readonly [K in keyof ID]: Index<ID[K]>;
} : never;
/**
 *
 * @param table The name of the DynamoDB table
 * @param config The dynamoDb and marshalling { @link Config }
 * @param client An optional implementation of the { @link DynamoDb } client
 */
export declare const Table: <T extends DynamoObject, PartitionKey extends string & keyof T, SortKey extends string & keyof T, Ixs extends IndexDefinitions = Record<string, never>>(tableDefintion: TableDefinition<T, PartitionKey, SortKey, Ixs>, config?: DynamodbTableConfig) => TableFactoryResult<T, PartitionKey, SortKey, Ixs>;
export {};
